/*
 * Provide a widget interface to humanclock.com
 * By Scott Pakin <scott+hc@pakin.org>
 */

var findURL_re;           // Matches an image URL within a jsclock.php page
var ajax_requests;        // Asynchronous HTTP request object
var timezone;             // Our computer's timezone (hours from UTC)
var cookie = "";          // Cookies received from humanclock.com
var clockSize;            // Clock size (1=small; 2=medium; 3=large)
var failedConnect = 0;    // 1=previous HTTP access failed; 0=succeeded


// Ensure that humanclock.com knows what timezone we're in then set
// up the widget to wait two minutes on the splash screen before
// loading an image page from humanclock.com and displaying the image
// to which it refers.
function initialize () {
  // Prepare to perform AJAX requests and parse the resulting HTML.
  findURL_re = new RegExp(/http:.*\.jpg/);
  ajax_requests = [new XMLHttpRequest(), new XMLHttpRequest()];

  // Set the current timezone and adjust the clock size.
  timezone = widget.preferenceForKey("timezone");
  cookie = widget.preferenceForKey("cookie");
  checkTimezone(timezone == "" || cookie == "");
  configureClock("http://www.humanclock.com/clock.php?cp=1&uo=2", null);  // Request random pictures -- does this work?
  setTimeout("requestFirstPage()", 2000);
  clockSize = widget.preferenceForKey("clockSize");
  if (clockSize == "")
    setClockSize(2, false);
  else
    setClockSize(parseInt(clockSize), false);
  configureButtons();
}


// Configure the "larger" and "smaller" buttons to respond to user
// input when efficacious.
function configureButtons () {
  // Assign dynamic styles to the "larger" button.
  var lbutton = document.getElementById("larger");
  lbutton.onmouseover = function () {
    if (clockSize != 3)
      lbutton.className = "hc-button-ready";
  };
  lbutton.onmousedown = function () {
    if (clockSize != 3)
      lbutton.src = "larger20x20-active.png";
  };
  lbutton.onmouseup = function () {
    if (clockSize != 3)
      lbutton.src = "larger20x20.png";
  };
  lbutton.onmouseout = function () {
    lbutton.className = "hc-button";
    lbutton.src = "larger20x20.png";
  };
  lbutton.onclick = function () {setClockSize(clockSize+1)};

  // Assign dynamic styles to the "smaller" button.
  var sbutton = document.getElementById("smaller");
  sbutton.onmouseover = function () {
    if (clockSize != 1)
      sbutton.className = "hc-button-ready";
  };
  sbutton.onmousedown = function () {
    if (clockSize != 1)
      sbutton.src = "smaller20x20-active.png";
  };
  sbutton.onmouseup = function () {
    if (clockSize != 1)
      sbutton.src = "smaller20x20.png";
  };
  sbutton.onmouseout = function () {
    sbutton.className = "hc-button";
    sbutton.src = "smaller20x20.png";
  };
  sbutton.onclick = function () {setClockSize(clockSize-1)};
}


// After showing the splash screen for 2 seconds, request the first
// page containing a human-clock image.
function requestFirstPage () {
  requestPage();
  setTimeout("requestRemainingPages()", 1000*(60-new Date().getSeconds()));
}


// Now that we've loaded the first page and displayed the first image,
// schedule all future updates on the minute.
function requestRemainingPages () {
  setInterval("requestPage()", 60000);
  requestPage();
}


// Request an updated image page from humanclock.com.
function requestPage () {
  document.getElementById("clock-picture").alt = "Loading...";
  checkTimezone(false);
  ajax_requests[0].open("GET", "http://www.humanclock.com/jsclock.php?uo=2&new=1&s="+clockSize.toString(), true);
  ajax_requests[0].onreadystatechange = updateImage;
  ajax_requests[0].setRequestHeader("Cookie", cookie);
  try {
    ajax_requests[0].send();
  }
  catch (err) {
    failedConnect = 1;
    displayTime();
  }
}


// Update the clock image as soon as the network transaction completes.
// If we received a cookie with the image, store the cookie.
function updateImage () {
  if (ajax_requests[0].readyState != 4)
    return;
  if (ajax_requests[0].status != 200
      || !ajax_requests[0].responseText) {
    failedConnect = 1;
    displayTime();
    return;
  }
  var newURL = findURL_re.exec(ajax_requests[0].responseText);
  document.getElementById("clock-picture").src = newURL;
  extractCookies(ajax_requests[0]);
  document.getElementById("main-view").style.display = "block";
  document.getElementById("local-clock").style.display = "none";
  document.getElementById("splash-screen").style.display = "none";
  document.getElementById("clock-picture").style.display = "inline";
  document.getElementById("clock-picture").alt = "Current time";

  // If we just regained access to humanclock.com, request a fresh
  // cookie just in case we need it.
  if (failedConnect) {
    failedConnect = 0;
    checkTimezone(true);
  }
}


// Locally display the time if we were unable to connect to
// humanclock.com.
function displayTime () {
  // Format the time on a 12-hour clock.
  var now = new Date();
  var hours = now.getHours();
  var minutes = now.getMinutes();
  if (hours > 12)
    hours -= 12;
  if (hours == 0)
    hours = 12;
  var timeString = hours + ":" + (minutes<10 ? "0" : "") + minutes;
  document.getElementById("time-string").innerText = timeString;

  // Display the local clock.
  document.getElementById("main-view").style.display = "block";
  document.getElementById("local-clock").style.display = "block";
  document.getElementById("splash-screen").style.display = "none";
  document.getElementById("clock-picture").style.display = "none";
}


// If our timezone has changed (e.g., because of daylight saving time)
// or if a timezone update is forced by the caller, notify
// humanclock.com and request a cookie representing the new timezone.
function checkTimezone (forceUpdate) {
  newTimezone = new Date().getTimezoneOffset() / -60.0;
  if (timezone != newTimezone || forceUpdate) {
    timezone = newTimezone;
    widget.setPreferenceForKey(timezone.toString(), "timezone");
    configureClock("http://www.humanclock.com/changemytimezonerightnowplease.php",
                   "btn_change=Set+clock+to+the+date%2Ftime+below&f_newOffset="+timezone.toString());
  }
}


// Send configuration parameters to humanclock.com (synchronously so
// as to limit the size of the ajax_requests[] array) and receive a
// cookie in return.
function configureClock (url, formData) {
  try {
    if (formData == null) {
      ajax_requests[1].open("GET", url, false);
      ajax_requests[1].onreadystatechange = receiveCookie;
      if (cookie != "")
        ajax_requests[1].setRequestHeader("Cookie", cookie);
      ajax_requests[1].send();
    }
    else {
      ajax_requests[1].open("POST", url, false);
      ajax_requests[1].onreadystatechange = receiveCookie;
      ajax_requests[1].setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
      if (cookie != "")
        ajax_requests[1].setRequestHeader("Cookie", cookie);
      ajax_requests[1].send(formData);
    }
  }
  catch (err) {
    failedConnect = 1;
  }
}


// Extract a cookie from the received page and store it with the widget.
function receiveCookie () {
  if (ajax_requests[1].readyState != 4)
    return;
  if (ajax_requests[1].status != 200
      || !ajax_requests[1].responseText) {
    failedConnect = 1;
    return;
  }
  extractCookies(ajax_requests[1]);
}


// Set the clock size to size 1, 2, or 3.
function setClockSize (newSize) {
  // Determine the new size to use.
  if (newSize < 1 || newSize > 3)
    return;
  clockSize = newSize;
  if (arguments.length < 2 || arguments[1] == true)
    requestPage();
  widget.setPreferenceForKey(clockSize, "clockSize");
  var newX, newY;
  var fontsize_title, fontsize_body;
  switch (clockSize) {
    case 1:
      newX = 155;
      newY = 125;
      fontsize_title = 16;
      fontsize_body = 9;
      break;

    case 2:
      newX = 260;
      newY = 201;
      fontsize_title = 32;
      fontsize_body = 14;
      break;

    case 3:
      newX = 450;
      newY = 354;
      fontsize_title = 48;
      fontsize_body = 24;
      break;

    default:
      return;
      break;
  }

  // Resize the window and various objects within it.
  doResize = function (DOMobj) {
    DOMobj.style.width = newX;
    DOMobj.style.height = newY;
  };
  document.getElementById("splash-title").style.fontSize = fontsize_title + "px";
  document.getElementById("splash-text").style.fontSize = fontsize_body + "px";
  document.getElementById("local-title").style.fontSize = fontsize_title + "px";
  document.getElementById("local-text").style.fontSize = fontsize_body*1.25 + "px";
  document.getElementById("time-string").style.fontSize = fontsize_title*2 + "px";
  doResize(document.getElementById("splash-screen"));
  doResize(document.getElementById("main-view"));
  doResize(document.getElementById("local-clock"));
  doResize(document.getElementById("clock-picture"));
  doResize(document.body);
  window.resizeTo(newX, newY);
}


// Given a completed AJAX request, extract and store any included cookies.
function extractCookies (req) {
  var newCookie = req.getResponseHeader("Set-Cookie");
  if (newCookie != "") {
    cookie = newCookie;
    widget.setPreferenceForKey(cookie, "cookie");
  }
}
